/**
 * JASMINe
 * Copyright (C) 2009 Bull S.A.S.
 * Contact: jasmine@ow2.org
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307
 * USA
 *
 * --------------------------------------------------------------------------
 * $Id: TestDbChecker.java 4798 2009-08-07 12:30:47Z alitokmen $
 * --------------------------------------------------------------------------
 */

package org.ow2.jasmine.monitoring.tests.timerchecker;

import java.util.Calendar;
import java.util.Date;
import java.util.Hashtable;

import javax.naming.Context;
import javax.naming.InitialContext;
import javax.naming.NamingException;

import org.apache.tools.ant.taskdefs.Sleep;
import org.ow2.jasmine.event.beans.JasmineEventEB;
import org.ow2.jasmine.event.messages.JasmineEventTimer;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventSLBRemote;
import org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventTimerSLBRemote;
import org.testng.Assert;
import org.testng.annotations.BeforeClass;
import org.testng.annotations.Test;

/**
 * This class is testing the Purge Timer module which can delete events from
 * database depending on criterias. User can define criterias like: - delete
 * events older than certain age - delete events older than Nth event - define a
 * timer to periodiacally execute the purge task.
 * @author Arda Aydin
 */
public class TestPurgeTimer {

    private static long SECOND_IN_MILISECOND = 1000;

    private static long MINUTE_IN_MILISECOND = SECOND_IN_MILISECOND * 60;

    private static long HOUR_IN_MILISECOND = MINUTE_IN_MILISECOND * 60;

    private static long DAY_IN_MILISECOND = HOUR_IN_MILISECOND * 24;

    private static long MONTH_IN_MILISECOND = DAY_IN_MILISECOND * 30;

    private static long YEAR_IN_MILISECOND = DAY_IN_MILISECOND * 365;

    private static final String mappedName = "org.ow2.jasmine.monitoring.eventswitch.beans.impl.JasmineEventSLBImpl_org.ow2.jasmine.monitoring.eventswitch.beans.JasmineEventSLBRemote@Remote";

    private JasmineEventTimerSLBRemote eventTimerBeanSLB = null;

    private JasmineEventSLBRemote eventBeanSLB = null;

    @BeforeClass
    public void getBean() throws NamingException {
        this.eventTimerBeanSLB = (JasmineEventTimerSLBRemote) getContext().lookup("db-ejb/timer");
        this.eventBeanSLB = (JasmineEventSLBRemote) getContext().lookup(mappedName);
    }

    @Test
    public void checkEvents1() {

        sendEvents();
        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertNotNull(events, "we should have events in the database");

    }

    @Test(dependsOnMethods = {"checkEvents1"})
    public void testConfigureTimer1() throws Exception {

        sendEvents();

        JasmineEventTimer event = new JasmineEventTimer();

        event.setMaxEntryAge(MONTH_IN_MILISECOND);
        event.setMaxEntryNumber(10);
        event.setExecutionStartDate(null);
        event.setExecutionStartHours(0);
        event.setExecutionInterval(0);
        event.setExecuteOnAllServers(true);
        event.setImmediateExecution(true);
        event.setDomainName(null);
        event.setServerName(null);

        eventTimerBeanSLB.configureTimer(event);

    }

    @Test(dependsOnMethods = {"testConfigureTimer1"})
    public void checkEvents2() {

        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertNull(events, "we should not have events in the database");

    }

    @Test(dependsOnMethods = {"checkEvents2"})
    public void testConfigureTimer2() throws Exception {

        sendEvents();

        JasmineEventTimer event = new JasmineEventTimer();

        event.setMaxEntryAge(MONTH_IN_MILISECOND);
        event.setMaxEntryNumber(10);
        event.setExecutionStartDate(null);
        event.setExecutionStartHours(0);
        event.setExecutionInterval(0);
        event.setExecuteOnAllServers(false);
        event.setImmediateExecution(true);
        event.setDomainName("domain2");
        event.setServerName("server2");

        eventTimerBeanSLB.configureTimer(event);

    }

    @Test(dependsOnMethods = {"testConfigureTimer2"})
    public void checkEvents3() {

        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertEquals(events.length, 2, "we should have 2 events in the database");

    }

    @Test(dependsOnMethods = {"checkEvents3"})
    public void testConfigureTimer3() throws Exception {

        JasmineEventTimer event = new JasmineEventTimer();

        event.setMaxEntryAge(YEAR_IN_MILISECOND * 5);
        event.setMaxEntryNumber(10);
        event.setExecutionStartDate(null);
        event.setExecutionStartHours(0);
        event.setExecutionInterval(0);
        event.setExecuteOnAllServers(false);
        event.setImmediateExecution(true);
        event.setDomainName("domain3");
        event.setServerName("server3");

        eventTimerBeanSLB.configureTimer(event);

    }

    @Test(dependsOnMethods = {"testConfigureTimer3"})
    public void checkEvents4() {

        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertEquals(events.length, 2, "we should have 2 events in the database");

    }

    @Test(dependsOnMethods = {"checkEvents4"})
    public void testConfigureTimer4() throws Exception {

        JasmineEventTimer event = new JasmineEventTimer();

        event.setMaxEntryAge(YEAR_IN_MILISECOND * 5);
        event.setMaxEntryNumber(1);
        event.setExecutionStartDate(null);
        event.setExecutionStartHours(0);
        event.setExecutionInterval(0);
        event.setExecuteOnAllServers(true);
        event.setImmediateExecution(true);

        eventTimerBeanSLB.configureTimer(event);

    }

    @Test(dependsOnMethods = {"testConfigureTimer4"})
    public void checkEvents5() {

        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertEquals(events.length, 1, "we should have 1 events in the database");

    }

    @Test(dependsOnMethods = {"checkEvents5"})
    public void testConfigureTimer5() throws Exception {

        JasmineEventTimer event = new JasmineEventTimer();

        event.setMaxEntryAge(SECOND_IN_MILISECOND);
        event.setMaxEntryNumber(0);
        event.setExecutionStartDate(null);
        event.setExecutionStartHours(0);
        event.setExecutionInterval(0);
        event.setExecuteOnAllServers(true);
        event.setImmediateExecution(true);

        eventTimerBeanSLB.configureTimer(event);

    }

    @Test(dependsOnMethods = {"testConfigureTimer5"})
    public void checkEvents6() {

        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
        Assert.assertNull(events, "we should not have events in the database");

    }

//    @Test(dependsOnMethods = {"checkEvents6"})
//    public void testConfigureTimer6() throws Exception {
//
//        sendEvents();
//
//        JasmineEventTimer event = new JasmineEventTimer();
//
//        event.setMaxEntryAge(SECOND_IN_MILISECOND);
//        event.setMaxEntryNumber(0);
//        event.setExecutionStartDate(Calendar.getInstance().getTime());
//        event.setExecutionStartHours(1);
//        event.setExecutionInterval(10000);
//        event.setExecuteOnAllServers(true);
//        event.setImmediateExecution(false);
//
//        eventTimerBeanSLB.configureTimer(event);
//
//    }
//
//    @Test(dependsOnMethods = {"testConfigureTimer6"})
//    public void checkEvents7() throws InterruptedException {
//
//        Thread.sleep(20000);
//
//        JasmineEventEB[] events = eventBeanSLB.getEvents(null, null, null, null, null, null, null, null, 10);
//        Assert.assertNull(events, "we should not have events in the database");
//
//    }

    protected Context getContext() throws NamingException {
        Hashtable<String, Object> env = new Hashtable<String, Object>();
        env.put(Context.INITIAL_CONTEXT_FACTORY, org.ow2.easybeans.component.smartclient.spi.SmartContextFactory.class
                .getName());
        env.put(Context.PROVIDER_URL, "smart://localhost:" + System.getProperty("smartFactory.port", "2503"));
        return new InitialContext(env);
    }

    private void sendEvents() {

        Calendar cal = Calendar.getInstance();
        Date nowDate = cal.getTime();
        Date oneYearAgoDate = new Date(nowDate.getTime() - YEAR_IN_MILISECOND);

        JasmineEventEB jEvent = new JasmineEventEB("domain1", "server1", "source1", "probe1", "value", oneYearAgoDate,
                "sname1");
        eventBeanSLB.saveEvent(jEvent);
        jEvent = new JasmineEventEB("domain2", "server2", "source1", "probe2", "value", oneYearAgoDate, "sname1");
        eventBeanSLB.saveEvent(jEvent);
        jEvent = new JasmineEventEB("domain3", "server3", "source1", "probe3", "value", oneYearAgoDate, "sname1");
        eventBeanSLB.saveEvent(jEvent);

    }

}